@@ -82,6 +82,7 @@ static const char* KEY_SHOW_WEEK     = "show_week_numbers";
 static const char *clock_config_tools [] = {
 	"system-config-date",
 	"redhat-config-date",
+        "/sbin/yast2 timezone",
 	"time-admin",
 };
 
@@ -216,19 +217,16 @@ clock_timeout_callback (gpointer data)
 	if (!cd->showseconds &&
 	    cd->format != CLOCK_FORMAT_UNIX &&
 	    cd->format != CLOCK_FORMAT_CUSTOM) {
-		if (cd->format != CLOCK_FORMAT_INTERNET && 
-		    new_time % 60 != cd->current_time % 60)
-		{
-				update_clock (cd);
+		if (cd->format == CLOCK_FORMAT_INTERNET && 
+		    (long)get_itime (new_time) !=
+		    (long)get_itime (cd->current_time)) {
+			update_clock (cd);
+		} else if ((cd->format == CLOCK_FORMAT_12 ||
+			    cd->format == CLOCK_FORMAT_24) &&
+			   new_time / 60 != cd->current_time / 60) {
+			update_clock (cd);
 		}
-		else if ((long)get_itime (new_time) !=
-			 (long)get_itime (cd->current_time))
-		{
-				update_clock (cd);
-		}
-	}
-	else
-	{
+	} else {
 		update_clock (cd);
 	}
 
@@ -250,16 +248,59 @@ get_itime (time_t current_time)
 	return itime;
 }
 
+/* adapted from panel-toplevel.c */
+static int
+calculate_minimum_height (GtkWidget        *widget,
+                          PanelAppletOrient orientation)
+{
+        PangoContext     *context;
+        PangoFontMetrics *metrics;
+        int               focus_width = 0;
+        int               focus_pad = 0;
+        int               ascent;
+        int               descent;
+        int               thickness;
+  
+        context = gtk_widget_get_pango_context (widget);
+        metrics = pango_context_get_metrics (context,
+                                             widget->style->font_desc,
+                                             pango_context_get_language (context));
+  
+        ascent  = pango_font_metrics_get_ascent  (metrics);
+        descent = pango_font_metrics_get_descent (metrics);
+  
+        pango_font_metrics_unref (metrics);
+
+        gtk_widget_style_get (widget,
+                              "focus-line-width", &focus_width,
+                              "focus-padding", &focus_pad,
+                              NULL);
+
+        if (orientation == PANEL_APPLET_ORIENT_UP
+            || orientation == PANEL_APPLET_ORIENT_DOWN) {
+                thickness = widget->style->ythickness;
+        } else {
+                thickness = widget->style->xthickness;
+        }
+
+        return PANGO_PIXELS (ascent + descent) + 2 * (focus_width + focus_pad + thickness);
+}
+
+static gboolean
+use_two_line_format (ClockData *cd)
+{
+        if (cd->size >= 2 * calculate_minimum_height (cd->applet, cd->orient))
+                return TRUE;
+
+        return FALSE;
+}
+
 static void
 update_timeformat (ClockData *cd)
 {
  /* Show date in another line if panel is vertical, or
   * horizontal but large enough to hold two lines of text
   */
-#define USE_TWO_LINE_FORMAT(cd) ((cd)->orient == PANEL_APPLET_ORIENT_LEFT  || \
-                                 (cd)->orient == PANEL_APPLET_ORIENT_RIGHT || \
-                                 (cd)->size >= GNOME_Vertigo_PANEL_MEDIUM)
-
 	const char *time_format;
 	const char *date_format;
 	char       *clock_format;
@@ -280,7 +321,7 @@ update_timeformat (ClockData *cd)
 		 */
 		date_format = _("%a %b %e");
 
-		if (USE_TWO_LINE_FORMAT (cd))
+		if (use_two_line_format (cd))
 			/* translators: reverse the order of these arguments
 			 *              if the time should come before the
 			 *              date on a clock in your locale.
@@ -303,8 +344,6 @@ update_timeformat (ClockData *cd)
 		cd->timeformat = g_strdup ("???");
 
 	g_free (clock_format);
-
-#undef USE_TWO_LINE_FORMAT
 }
 
 /* sets accessible name and description for the widget */
@@ -341,9 +380,7 @@ update_clock (ClockData * cd)
 		tm = localtime (&cd->current_time);
 
 	if (cd->format == CLOCK_FORMAT_UNIX) {
-		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
-		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
+		if (use_two_line_format (cd)) {
 			g_snprintf (hour, sizeof(hour), "%lu\n%05lu",
 				    (unsigned long)(cd->current_time / 100000L),
 				    (unsigned long)(cd->current_time % 100000L));
@@ -375,17 +412,25 @@ update_clock (ClockData * cd)
 
         gtk_widget_queue_resize (cd->toggle);
 
-	/* Show date in tooltip */
-	loc = g_locale_from_utf8 (_("%A %B %d"), -1, NULL, NULL, NULL);
-	if (!loc)
-		strcpy (date, "???");
-	else if (strftime (date, sizeof (date), loc, tm) <= 0)
-		strcpy (date, "???");
-	g_free (loc);
+        if (!cd->showdate) {
+                /* Show date in tooltip */
+                loc = g_locale_from_utf8 (_("%A %B %d"), -1, NULL, NULL, NULL);
+                if (!loc)
+                        strcpy (date, "???");
+                else if (strftime (date, sizeof (date), loc, tm) <= 0)
+                        strcpy (date, "???");
+                g_free (loc);
 
-	utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
-	set_tooltip (cd->applet, cd->toggle, utf8);
-	g_free (utf8);
+                utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
+                set_tooltip (cd->applet, cd->toggle, utf8);
+                g_free (utf8);
+        } else {
+#ifdef HAVE_LIBECAL
+                set_tooltip (cd->applet, cd->toggle, _("Click to view your appointments and tasks"));
+#else
+                set_tooltip (cd->applet, cd->toggle, _("Click to view month calendar"));
+#endif
+        }
 }
 
 static void
@@ -549,6 +594,7 @@ enum {
         TASK_COLUMN_OVERDUE_ATTR,
         TASK_COLUMN_COLOR,
         TASK_COLUMN_URL,
+        TASK_COLUMN_PRIORITY,
         N_TASK_COLUMNS
 };
 
@@ -629,6 +675,7 @@ handle_tasks_changed (ClockData *cd)
                                     TASK_COLUMN_COMPLETED_TIME,        task->completed_time,
                                     TASK_COLUMN_COLOR,                 task->color_string,
                                     TASK_COLUMN_URL,                   task->url,
+                                    TASK_COLUMN_PRIORITY,              task->priority,
                                     -1);
 
                 g_free (percent_complete_text);
@@ -809,17 +856,24 @@ task_activated_cb (GtkTreeView       *view,
                    GtkTreeViewColumn *column,
                    ClockData         *cd)
 {
-        GtkTreeIter iter;
-        char       *uri;
+        GtkTreeIter  iter;
+        GtkTreePath *child_path;
+        char        *uri;
 
-        gtk_tree_model_get_iter (GTK_TREE_MODEL (cd->tasks_model), &iter, path);
+        child_path = gtk_tree_model_filter_convert_path_to_child_path (cd->tasks_filter, path);
+
+        gtk_tree_model_get_iter (GTK_TREE_MODEL (cd->tasks_model), &iter, child_path);
         gtk_tree_model_get (GTK_TREE_MODEL (cd->tasks_model), &iter,
                             TASK_COLUMN_URL, &uri, -1);
 
-        if (uri)
+        if (uri) {
                 gnome_url_show_on_screen (uri,
 					  gtk_widget_get_screen (cd->applet),
 					  NULL);
+        }
+
+        g_free (uri);
+        gtk_tree_path_free (child_path);
 
         return TRUE;
 }
@@ -876,6 +930,33 @@ appointment_pixbuf_cell_data_func (GtkTreeViewColumn *column,
                                              APPOINTMENT_COLUMN_COLOR);
 }
 
+static int
+compare_priority  (GtkTreeModel *model,
+                   GtkTreeIter  *a,
+                   GtkTreeIter  *b,
+                   gpointer      user_data)
+{
+        int priority_a;
+        int priority_b;
+
+        gtk_tree_model_get (model, a, TASK_COLUMN_PRIORITY, &priority_a, -1);
+        gtk_tree_model_get (model, b, TASK_COLUMN_PRIORITY, &priority_b, -1);
+
+	/* We change undefined priorities so they appear after 'Low'. */
+	if (priority_a <= 0)
+		priority_a = 10;
+	if (priority_b <= 0)
+		priority_b = 10;
+
+	/* We'll just use the ordering of the priority values. */
+	if (priority_a < priority_b)
+		return -1;
+	else if (priority_a > priority_b)
+		return 1;
+	else
+		return 0;
+}
+
 static GtkWidget *
 create_task_list (ClockData  *cd,
                   GtkWidget **tree_view,
@@ -911,11 +992,21 @@ create_task_list (ClockData  *cd,
                         G_TYPE_LONG,           /* completed time          */
                         PANGO_TYPE_ATTR_LIST,  /* summary text attributes */
                         G_TYPE_STRING,         /* color                   */
-                        G_TYPE_STRING          /* url                     */
+                        G_TYPE_STRING,         /* url                     */
+                        G_TYPE_INT             /* priority                */
                 };
 
                 cd->tasks_model = gtk_list_store_newv (N_TASK_COLUMNS, column_types);
 
+                gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (cd->tasks_model),
+                                                 TASK_COLUMN_PRIORITY,
+                                                 compare_priority,
+                                                 NULL, NULL);
+
+                gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (cd->tasks_model),
+                                                      TASK_COLUMN_PRIORITY,
+                                                      GTK_SORT_ASCENDING);
+
                 cd->tasks_filter = GTK_TREE_MODEL_FILTER (
                         gtk_tree_model_filter_new (GTK_TREE_MODEL (cd->tasks_model),
                                                    NULL));
@@ -933,8 +1024,6 @@ create_task_list (ClockData  *cd,
                          NULL);
         }
 
-        /* FIXME: implement sorting */
-
         *tree_view = view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (cd->tasks_filter));
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
 
@@ -1546,7 +1635,6 @@ create_clock_widget (ClockData *cd)
 				  G_CALLBACK (unfix_size),
 				  cd);
 	gtk_label_set_justify (GTK_LABEL (clock), GTK_JUSTIFY_CENTER);
-	gtk_label_set_line_wrap (GTK_LABEL (clock), TRUE);
 	gtk_widget_show (clock);
 
 	toggle = gtk_toggle_button_new ();
@@ -1576,6 +1664,15 @@ create_clock_widget (ClockData *cd)
 	cd->props = NULL;
 
 	cd->orient = panel_applet_get_orient (PANEL_APPLET (cd->applet));
+
+	/* Initialize label orientation */
+	if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
+		gtk_label_set_angle (GTK_LABEL (cd->clockw), 270);
+	else if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+		gtk_label_set_angle (GTK_LABEL (cd->clockw), 90);
+	else
+		gtk_label_set_angle (GTK_LABEL (cd->clockw), 0);
+
 	cd->size = panel_applet_get_size (PANEL_APPLET (cd->applet));
 
 	g_signal_connect (G_OBJECT(clock), "destroy",
@@ -1596,10 +1693,17 @@ applet_change_orient (PanelApplet       *applet,
 		      PanelAppletOrient  orient,
 		      ClockData         *cd)
 {
-	cd->orient = orient;
-
+        cd->orient = orient;
+        
+        if (cd->orient == PANEL_APPLET_ORIENT_LEFT)
+                gtk_label_set_angle (GTK_LABEL (cd->clockw), 270);
+        else if (cd->orient == PANEL_APPLET_ORIENT_RIGHT)
+                gtk_label_set_angle (GTK_LABEL (cd->clockw), 90);
+        else
+                gtk_label_set_angle (GTK_LABEL (cd->clockw), 0);
+	
         unfix_size (cd);
-	update_clock (cd);
+        update_clock (cd);
         update_popup (cd);
 }
 
@@ -1617,7 +1721,7 @@ applet_change_background (PanelApplet               *applet,
 	gtk_widget_set_style (GTK_WIDGET (cd->applet), NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (GTK_WIDGET (cd->applet), rc_style);
-	g_object_unref (rc_style);
+	gtk_rc_style_unref (rc_style);
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
@@ -1632,6 +1736,7 @@ applet_change_background (PanelApplet               *applet,
 			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
 		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
 		gtk_widget_set_style (GTK_WIDGET (cd->applet), style);
+		g_object_unref (style);
 		break;
 	}
 }
@@ -1741,25 +1846,38 @@ try_config_tool (GdkScreen  *screen,
 {
 	GtkWidget *dialog;
 	GError    *err;
-	char      *argv[2];
-	char      *app;
+	char     **argv;
+        char      *path;
 
-	g_return_val_if_fail (tool != NULL, FALSE);
+        if (!tool || tool[0] == '\0')
+                return FALSE;
 
-	app = g_find_program_in_path (tool);
+        if (!g_shell_parse_argv (tool, NULL, &argv, NULL))
+                return FALSE;
 
-	if (!app)
-		return FALSE;
-		
-	argv [0] = app;
-	argv [1] = NULL;		
+        if (!(path = g_find_program_in_path (argv [0]))) {
+                g_strfreev (argv);
+                return FALSE;
+        }
+
+        g_free (path);
 
 	err = NULL;
-	if (gdk_spawn_on_screen (screen, NULL, argv, NULL, 0, NULL, NULL, NULL, &err)) {
-		g_free (app);
+	if (gdk_spawn_on_screen (screen,
+                                 NULL,
+                                 argv,
+                                 NULL,
+                                 G_SPAWN_SEARCH_PATH,
+                                 NULL,
+                                 NULL,
+                                 NULL,
+                                 &err)) {
+                g_strfreev (argv);
 		return TRUE;
 	}
-			
+
+        g_strfreev (argv);
+
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
@@ -1774,8 +1892,6 @@ try_config_tool (GdkScreen  *screen,
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 			
 	gtk_widget_show_all (dialog);			
-
-	g_free (app);
 		
 	return TRUE;
 }
@@ -1791,8 +1907,7 @@ config_date (BonoboUIComponent *uic,
 
 	screen = gtk_widget_get_screen (cd->applet);
 
-	if (cd->config_tool && cd->config_tool[0] &&
-            try_config_tool (screen, cd->config_tool))
+	if (try_config_tool (screen, cd->config_tool))
 		return;
 	
 	for (i = 0; i < G_N_ELEMENTS (clock_config_tools); i++)
@@ -1900,6 +2015,30 @@ gmt_time_changed (GConfClient  *client,
 	refresh_clock_timeout (clock);
 }
 
+static gboolean
+check_config_tool_command (const char *config_tool)
+{
+        char **argv;
+        char  *path;
+
+        if (!config_tool || config_tool[0] == '\0')
+                return FALSE;
+
+        argv = NULL;
+        if (!g_shell_parse_argv (config_tool, NULL, &argv, NULL))
+                return FALSE;
+
+        if (!(path = g_find_program_in_path (argv [0]))) {
+                g_strfreev (argv);
+                return FALSE;
+        }
+
+        g_free (path);
+        g_strfreev (argv);
+
+        return TRUE;
+}
+
 static void
 config_tool_changed (GConfClient  *client,
                      guint         cnxn_id,
@@ -1913,8 +2052,10 @@ config_tool_changed (GConfClient  *client,
 
 	value = gconf_value_get_string (entry->value);
 
-        g_free (clock->config_tool);
-	clock->config_tool = g_strdup (value);
+        if (check_config_tool_command (value)) {
+                g_free (clock->config_tool);
+                clock->config_tool = g_strdup (value);
+        }
 }
 
 static void
@@ -2087,7 +2228,6 @@ fill_clock_applet (PanelApplet *applet)
 	GError            *error;
         int                format;
 	char              *format_str;
-	char              *config_tool;
 	
 	panel_applet_add_preferences (applet, "/schemas/apps/clock_applet/prefs", NULL);
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
@@ -2158,7 +2298,6 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (applet_change_pixel_size),
 			  cd);
 
-	/* FIXME: initial background, this needs some panel-applet voodoo */
 	g_signal_connect (G_OBJECT (cd->applet),
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
@@ -2184,26 +2323,26 @@ fill_clock_applet (PanelApplet *applet)
 					      NULL);
 	}
 
-        config_tool = NULL;
-	if (cd->config_tool && cd->config_tool [0])
-		config_tool = g_find_program_in_path (cd->config_tool);
+        if (!check_config_tool_command (cd->config_tool)) {
+                g_free (cd->config_tool);
+                cd->config_tool = NULL;
+        }
 
-        if (!config_tool) {
+        if (!cd->config_tool) {
                 int i;
 
                 for (i = 0; i < G_N_ELEMENTS (clock_config_tools); i++)
-                        if ((config_tool = g_find_program_in_path (clock_config_tools [i])))
+                        if (check_config_tool_command (clock_config_tools [i])) {
+                                cd->config_tool = g_strdup (clock_config_tools [i]);
                                 break;
+                        }
         }
 
-	if (!config_tool)
-		bonobo_ui_component_set_prop (popup_component,
-					      "/commands/ClockConfig",
-					      "hidden", "1",
-					      NULL);
-        
-        g_free (config_tool);
-	
+        bonobo_ui_component_set_prop (popup_component,
+                                      "/commands/ClockConfig",
+                                      "hidden", cd->config_tool ? "0" : "1",
+                                      NULL);
+
 	return TRUE;
 }
 
@@ -2580,8 +2719,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		"Dan Mueth <d-mueth@uchicago.edu>",
 		NULL
 	};
-	/* Translator credits */
-	const char *translator_credits = _("translator-credits");
 
 	if (cd->about) {
 		gtk_window_set_screen (GTK_WINDOW (cd->about),
@@ -2598,8 +2735,8 @@ display_about_dialog (BonoboUIComponent *uic,
 		      "comments", _("The Clock displays the current time and date"),
 		      "authors", authors,
 		      "documenters", documenters,
-		      "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-		      "logo_icon_name", "clock",
+		      "translator-credits", _("translator-credits"),
+		      "logo-icon-name", "clock",
 		      NULL);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (cd->about), "clock", "Clock");
