@@ -59,15 +59,16 @@ typedef struct
 } SystemTray;
 
 static void
-help_cb (PanelApplet *applet,
-         void        *data)
+help_cb (BonoboUIComponent *uic,
+	 SystemTray        *tray,	  
+	 const gchar       *verbname)   
 {
   GError *err;
 
   err = NULL;  
   gnome_help_display_desktop_on_screen (NULL, "user-guide",
                                         "user-guide.xml", "gospanel-567",
-					gtk_widget_get_screen (GTK_WIDGET (applet)),
+					gtk_widget_get_screen (GTK_WIDGET (tray->applet)),
 					&err);
 
   if (err != NULL)
@@ -87,7 +88,7 @@ help_cb (PanelApplet *applet,
       
       gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
       gtk_window_set_screen (GTK_WINDOW (dialog),
-                             gtk_widget_get_screen (GTK_WIDGET (applet)));
+                             gtk_widget_get_screen (GTK_WIDGET (tray->applet)));
       
       gtk_widget_show (dialog);
 
@@ -112,7 +113,6 @@ about_cb (BonoboUIComponent *uic,
     "Sun GNOME Documentation Team <gdocteam@sun.com>",
     NULL
   };
-  const char *translator_credits = _("translator-credits");
 
   screen = gtk_widget_get_screen (GTK_WIDGET (tray->applet));
 
@@ -125,13 +125,13 @@ about_cb (BonoboUIComponent *uic,
 
   tray->about_dialog = gtk_about_dialog_new ();
   g_object_set (tray->about_dialog,
-                "name", _("Panel Notification Area"),
+                "name", _("Notification Area"),
                 "version", VERSION,
                 "copyright", "Copyright \xc2\xa9 2002 Red Hat, Inc.",
                 "authors", (const char **) authors,
                 "documenters", (const char **) documenters,
-                "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-                "logo_icon_name", "panel-notification-area",
+                "translator-credits", _("translator-credits"),
+                "logo-icon-name", "panel-notification-area",
                 NULL);
   
   gtk_window_set_screen (GTK_WINDOW (tray->about_dialog), screen);
@@ -251,7 +251,7 @@ applet_change_background (PanelApplet               *applet,
   gtk_widget_set_style (GTK_WIDGET (tray->applet), NULL);
   rc_style = gtk_rc_style_new ();
   gtk_widget_modify_style (GTK_WIDGET (tray->applet), rc_style);
-  g_object_unref (rc_style);
+  gtk_rc_style_unref (rc_style);
 
   switch (type)
     {
@@ -268,6 +268,7 @@ applet_change_background (PanelApplet               *applet,
 
       style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
       gtk_widget_set_style (GTK_WIDGET (tray->applet), style);
+      g_object_unref (style);
       break;
     }
 }
