@@ -34,6 +34,7 @@
 
 
 static gboolean panel_background_composite (PanelBackground *background);
+static void load_background_file (PanelBackground *background);
 
 
 static void
@@ -97,10 +98,16 @@ panel_background_prepare (PanelBackground *background)
 
 	switch (effective_type) {
 	case PANEL_BACK_NONE:
-		if (background->default_pixmap)
-			gdk_window_set_back_pixmap (
-				background->window, background->default_pixmap, FALSE);
-		else
+		if (background->default_pixmap) {
+			if (background->default_pixmap != (GdkPixmap*) GDK_PARENT_RELATIVE)
+				gdk_window_set_back_pixmap (background->window,
+							    background->default_pixmap,
+							    FALSE);
+			else
+				gdk_window_set_back_pixmap (background->window,
+							    NULL,
+							    TRUE);
+		} else
 			gdk_window_set_background (
 				background->window, &background->default_color);
 		break;
@@ -348,6 +355,7 @@ get_scaled_and_rotated_pixbuf (PanelBackground *background)
 	int        panel_width, panel_height;
 	int        width, height;
 
+	load_background_file (background);
 	if (!background->loaded_image)
 		return NULL;
 
@@ -548,6 +556,17 @@ panel_background_set_type (PanelBackground     *background,
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_gdk_color_no_update (PanelBackground *background,
+					  GdkColor        *gdk_color)
+{
+	g_return_if_fail (gdk_color != NULL);
+
+	background->color.gdk.red   = gdk_color->red;
+	background->color.gdk.green = gdk_color->green;
+	background->color.gdk.blue  = gdk_color->blue;
+}
+
 void
 panel_background_set_gdk_color (PanelBackground *background,
 				GdkColor        *gdk_color)
@@ -560,14 +579,18 @@ panel_background_set_gdk_color (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	background->color.gdk.red   = gdk_color->red;
-	background->color.gdk.green = gdk_color->green;
-	background->color.gdk.blue  = gdk_color->blue;
-
+	panel_background_set_gdk_color_no_update (background, gdk_color);
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_opacity_no_update (PanelBackground *background,
+				        guint16          opacity)
+{
+	background->color.alpha = opacity;
+	panel_background_update_has_alpha (background);
+}
+
 void
 panel_background_set_opacity (PanelBackground *background,
 			      guint16          opacity)
@@ -576,22 +599,53 @@ panel_background_set_opacity (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	background->color.alpha = opacity;
-
-	panel_background_update_has_alpha (background);
-
+	panel_background_set_opacity_no_update (background, opacity);
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_color_no_update (PanelBackground *background,
+				      PanelColor      *color)
+{
+	g_return_if_fail (color != NULL);
+
+	panel_background_set_gdk_color_no_update (background, &(color->gdk));
+	panel_background_set_opacity_no_update (background, color->alpha);
+}
+
 void
 panel_background_set_color (PanelBackground *background,
 			    PanelColor      *color)
 {
 	g_return_if_fail (color != NULL);
 
-	panel_background_set_gdk_color (background, &(color->gdk));
-	panel_background_set_opacity (background, color->alpha);
+	if (background->color.gdk.red   == color->gdk.red &&
+	    background->color.gdk.green == color->gdk.green &&
+	    background->color.gdk.blue  == color->gdk.blue &&
+	    background->color.alpha  == color->alpha)
+		return;
+
+	free_transformed_resources (background);
+	panel_background_set_color_no_update (background, color);
+	panel_background_transform (background);
+}
+
+static void
+panel_background_set_image_no_update (PanelBackground *background,
+				      const char      *image)
+{
+	if (background->loaded_image)
+		g_object_unref (background->loaded_image);
+	background->loaded_image = NULL;
+
+	if (background->image)
+		g_free (background->image);
+	background->image = NULL;
+
+	if (image && image [0])
+		background->image = g_strdup (image);
+
+	panel_background_update_has_alpha (background);
 }
 
 void
@@ -605,25 +659,17 @@ panel_background_set_image (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	if (background->loaded_image)
-		g_object_unref (background->loaded_image);
-	background->loaded_image = NULL;
-
-	if (background->image)
-		g_free (background->image);
-	background->image = NULL;
-
-	if (image && image [0]) {
-		background->image = g_strdup (image);
-		load_background_file (background);
-	}
-
-	panel_background_update_has_alpha (background);
-
+	panel_background_set_image_no_update (background, image);
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_fit_no_update (PanelBackground *background,
+				    gboolean         fit_image)
+{
+	background->fit_image = fit_image != FALSE;
+}
+
 void
 panel_background_set_fit (PanelBackground *background,
 			  gboolean         fit_image)
@@ -634,12 +680,17 @@ panel_background_set_fit (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	background->fit_image = fit_image;
-
+	panel_background_set_fit_no_update (background, fit_image);
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_stretch_no_update (PanelBackground *background,
+				        gboolean         stretch_image)
+{
+	background->stretch_image = stretch_image != FALSE;
+}
+
 void
 panel_background_set_stretch (PanelBackground *background,
 			      gboolean         stretch_image)
@@ -650,12 +701,17 @@ panel_background_set_stretch (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	background->stretch_image = stretch_image;
-
+	panel_background_set_stretch_no_update (background, stretch_image);
 	panel_background_transform (background);
 }
 
+static void
+panel_background_set_rotate_no_update (PanelBackground *background,
+				       gboolean         rotate_image)
+{
+	background->rotate_image = rotate_image != FALSE;
+}
+
 void
 panel_background_set_rotate (PanelBackground *background,
 			     gboolean         rotate_image)
@@ -666,9 +722,7 @@ panel_background_set_rotate (PanelBackground *background,
 		return;
 
 	free_transformed_resources (background);
-
-	background->rotate_image = rotate_image;
-
+	panel_background_set_rotate_no_update (background, rotate_image);
 	panel_background_transform (background);
 }
 
@@ -681,11 +735,11 @@ panel_background_set (PanelBackground     *background,
 		      gboolean             stretch_image,
 		      gboolean             rotate_image)
 {
-	panel_background_set_color (background, color);
-	panel_background_set_image (background, image);
-	panel_background_set_fit (background, fit_image);
-	panel_background_set_stretch (background, stretch_image);
-	panel_background_set_rotate (background, rotate_image);
+	panel_background_set_color_no_update (background, color);
+	panel_background_set_image_no_update (background, image);
+	panel_background_set_fit_no_update (background, fit_image);
+	panel_background_set_stretch_no_update (background, stretch_image);
+	panel_background_set_rotate_no_update (background, rotate_image);
 	panel_background_set_type (background, type);
 }
 
@@ -698,10 +752,11 @@ panel_background_set_default_style (PanelBackground *background,
 
 	background->default_color = *color;
 
-	if (pixmap)
+	if (pixmap && pixmap != (GdkPixmap*) GDK_PARENT_RELATIVE)
 		g_object_ref (pixmap);
 
-	if (background->default_pixmap)
+	if (background->default_pixmap
+	    && background->default_pixmap != (GdkPixmap*) GDK_PARENT_RELATIVE)
 		g_object_unref (background->default_pixmap);
 
 	background->default_pixmap = pixmap;
@@ -885,6 +940,10 @@ panel_background_free (PanelBackground *background)
 		g_object_unref (background->loaded_image);
 	background->loaded_image = NULL;
 
+	if (background->monitor)
+		g_object_unref (background->monitor);
+	background->monitor = NULL;
+
 	if (background->window)
 		g_object_unref (background->window);
 	background->window = NULL;
@@ -897,7 +956,8 @@ panel_background_free (PanelBackground *background)
 		g_object_unref (background->gc);
 	background->gc = NULL;
 
-	if (background->default_pixmap)
+	if (background->default_pixmap
+	    && background->default_pixmap != (GdkPixmap*) GDK_PARENT_RELATIVE)
 		g_object_unref (background->default_pixmap);
 	background->default_pixmap = NULL;
 }
@@ -907,10 +967,15 @@ panel_background_make_string (PanelBackground *background,
 			      int              x,
 			      int              y)
 {
-	char *retval = NULL;
+	PanelBackgroundType  effective_type;
+	char                *retval;
+	
+	retval = NULL;
 
-	if (background->type == PANEL_BACK_IMAGE ||
-	    (background->type == PANEL_BACK_COLOR && background->has_alpha)) {
+	effective_type = panel_background_effective_type (background);
+
+	if (effective_type == PANEL_BACK_IMAGE ||
+	    (effective_type == PANEL_BACK_COLOR && background->has_alpha)) {
 		GdkNativeWindow pixmap_xid;
 
 		if (!background->pixmap)
@@ -921,7 +986,7 @@ panel_background_make_string (PanelBackground *background,
 
 		retval = g_strdup_printf ("pixmap:%d,%d,%d", pixmap_xid, x, y);
 
-	} else if (background->type == PANEL_BACK_COLOR)
+	} else if (effective_type == PANEL_BACK_COLOR)
 		retval = g_strdup_printf (
 				"color:%.4x%.4x%.4x",
 				background->color.gdk.red,
