@@ -79,6 +79,9 @@ orientation_change (AppletInfo  *info,
 	case PANEL_OBJECT_ACTION:
 		button_widget_set_orientation (BUTTON_WIDGET (info->widget), orientation);
 		break;
+	case PANEL_OBJECT_MENU_BAR:
+		panel_menu_bar_set_orientation (PANEL_MENU_BAR (info->widget), orientation);
+		break;
 	case PANEL_OBJECT_DRAWER: {
 		Drawer      *drawer = info->data;
 		PanelWidget *panel_widget;
@@ -189,10 +192,8 @@ panel_back_change (GtkWidget *widget, gpointer data)
 static void
 panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 {
-	PanelToplevel *toplevel;
 	AppletInfo    *info;
 
-	toplevel = PANEL_WIDGET (widget)->toplevel;
 	info = g_object_get_data (G_OBJECT (applet), "applet_info");
 
 	orientation_change(info,PANEL_WIDGET(widget));
@@ -257,10 +258,6 @@ static void
 panel_destroy (PanelToplevel *toplevel,
 	       PanelData     *pd)
 {
-	PanelWidget *panel_widget;
-
-	panel_widget = panel_toplevel_get_panel_widget (toplevel);
-
 	panel_lockdown_notify_remove (G_CALLBACK (panel_recreate_context_menu),
 				      pd);
 
@@ -1215,13 +1212,10 @@ panel_delete_without_query (PanelToplevel *toplevel)
 	if (panel_toplevel_get_is_attached (toplevel) &&
 	    panel_widget->master_widget) {
 		AppletInfo *info;
-		Drawer *drawer;
 
 		info = g_object_get_data (G_OBJECT (panel_widget->master_widget),
 					  "applet_info");
 
-		drawer = (Drawer *) info->data;
-
 		panel_profile_delete_object (info);
 	} else
 		panel_profile_delete_toplevel (toplevel);
@@ -1344,7 +1338,5 @@ panel_delete (PanelToplevel *toplevel)
 		return;
 	}
 
-	panel_toplevel_push_autohide_disabler (toplevel);
-
 	panel_query_deletion (toplevel);
 }
